const mode = window.acquireVsCodeApi === undefined ? 'debug' : 'release';
console.log('digital-vcd-viewer mode: ' + mode);

let vscode = window.acquireVsCodeApi === undefined ? undefined : acquireVsCodeApi();

import { globalLookup, globalSetting } from '@/hook/global';
import { makeSaveViewPayload } from '@/hook/recover';
import axios from 'axios';

export let saveDelay = 1000;

/**
 * 
 * @param {string} originVcdFile 
 * @param {string} originVcdViewFile 
 * @param {any} payload 
 */
export async function saveView(originVcdFile, originVcdViewFile, payload) {
    if (payload === undefined) {
        return;
    }
    if (mode === 'debug') {
        const res = await axios.post('http://localhost:3000/save-view', { originVcdFile, originVcdViewFile, payload });
        // const res = await axios.get('http://localhost:3000');
        console.log(res);
    } else {
        vscode.postMessage({
            command: 'save-view',
            data: { originVcdFile, originVcdViewFile, payload: structuredClone(payload) }
        });
    }
}

/**
 * 
 * @param {string} originVcdFile
 * @param {string} originVcdViewFile 
 * @param {*} payload 
 */
export async function saveViewAs(originVcdFile, originVcdViewFile, payload) {
    if (mode === 'debug') {
        const res = await axios.post('http://localhost:3000/save-view-as', { originVcdFile, originVcdViewFile, payload });
        if (res.data && res.data.viewPath) {
            const newVcdViewPath = res.data.viewPath;
            globalLookup.originVcdViewFile = newVcdViewPath;
        }
    } else {
        vscode.postMessage({
            command: 'save-view-as',
            data: { originVcdFile, originVcdViewFile, payload: structuredClone(payload) }
        });
    }
}

export async function loadView(originVcdFile) {
    if (mode === 'debug') {
        const res = await axios.post('http://localhost:3000/load-view', { originVcdFile });
        if (res.data && res.data.recoverJson && res.data.viewPath) {
            globalLookup.originVcdViewFile = res.data.viewPath;
            return res.data.recoverJson;
        }
    } else {
        vscode.postMessage({
            command: 'load-view',
            data: { originVcdFile }
        });
    }

    return undefined;
}

/**
 * 
 * @param {number} delay 
 * @returns {(config: import('@/hook/recover').SavePayloadConfig) => void}
 */
function debounceSaveView(delay) {
    let timer;
    const configPool = {};
    return function (config) {
        if (!globalSetting.autoSaveView) {
            return;
        }
        // 记录所有的 payload tag
        Object.assign(configPool, config);

        if (timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(() => {
            const payload = makeSaveViewPayload(configPool);            
            const originVcdFile = globalLookup.originVcdFile;
            const originVcdViewFile = globalLookup.originVcdViewFile;
            saveView(originVcdFile, originVcdViewFile, payload);
        }, delay);
    }
}

export const saveViewApi = debounceSaveView(saveDelay);