import { globalSetting } from '../global';

import xOffsetUpdate from './x-offset-update';
import yOffsetUpdate from './y-offset-update';

import xScaleUpdate from './x-scale-update';


const xScaleLevels = {
    1: 10 / 9,
    2: 10 / 7,
    3: 10 / 6,
    4: 10 / 5,
    5: 10 / 4
};

// TODO: 优化横向滚动体验
const xOffsetLevel = {
    1: 0.035,
    2: 0.1,
    3: 0.2,
    4: 0.4,
    5: 0.5
};

const yOffsetLevel = {
    1: 20,
    2: 40,
    3: 80,
    4: 160,
    5: 320
};

const scaleUp = {
    desc: 'Zoom in time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScale * xScaleLevels[globalSetting.HorizontalScalingRatio])
};

const scaleDown = {
    desc: 'Zoom out time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScale / xScaleLevels[globalSetting.HorizontalScalingRatio])
};

const fullo = {
    desc: 'All of time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScaleMin)
};

const scroll = {
    left: {
        desc: 'Scroll into the past',
        fn: pstate => {
            const offset = xOffsetLevel[globalSetting.HorizontalRollRatio] * pstate.width;
            // console.log(`change from ${pstate.xOffset} to ${pstate.xOffset + pstate.time / xOffsetLevel[globalSetting.HorizontalRollRatio]}`);
            return xOffsetUpdate(pstate, () => pstate.xOffset + offset);
        }
    },
    right: {
        desc: 'Scroll into the future',
        fn: pstate => {
            // console.log(`change from ${pstate.xOffset} to ${pstate.xOffset - pstate.time / xOffsetLevel[globalSetting.HorizontalRollRatio]}`);
            const offset = xOffsetLevel[globalSetting.HorizontalRollRatio] * pstate.width;
            return xOffsetUpdate(pstate, () => pstate.xOffset - offset)
        }
    },
    up: {
        desc: 'scroll up',
        fn: pstate => yOffsetUpdate(pstate, () => pstate.yOffset - yOffsetLevel[globalSetting.VerticalRollRario])
    },
    down: {
        desc: 'scroll down',
        fn: pstate => yOffsetUpdate(pstate, () => pstate.yOffset + yOffsetLevel[globalSetting.VerticalRollRario])
    }
};

const editable = {
    desc: 'Toggle edit mode',
    fn: (pstate, cm) => {
        console.log('editable', pstate, cm);
    }
};

export default {
    // wheel
    scrollLeft: scroll.left,
    scrollRight: scroll.right,
    scrollUp: scroll.up,
    scrollDown: scroll.down,
    scaleUp: scaleUp,
    scaleDown: scaleDown,

    // key
    

    nop: { fn: () => false }
};
