// 鼠标滚轮事件

/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<string, Handler>} eventHandler 
 * @param {WheelEvent} event 
 */
function windowsWheel(element, pstate, deso, eventHandler, event) {
    if (pstate === undefined)  {
        pstate = deso.pstate;
    }
    const { deltaX, deltaY } = event;
    if (event.ctrlKey) {
        const key = (deltaY < 0)
            ? 'scaleUp'
            : ((deltaY > 0) ? 'scaleDown' : 'nop');
                
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (event.shiftKey) {
        const key = (deltaY < 0) ? 'scrollLeft' : ((deltaY > 0) ? 'scrollRight' : 'nop');
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (deltaX !== 0 && deltaY === 0) {
        const key = (deltaX < 0) ? 'scrollLeft' : 'scrollRight';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (deltaX === 0 && deltaY !== 0) {
        const key = (deltaY < 0) ? 'scrollUp' : 'scrollDown';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    }
}


/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<string, Handler>} eventHandler 
 * @param {WheelEvent} event 
 */
function macOsWheel(element, pstate, deso, eventHandler, event) {
    if (pstate === undefined)  {
        pstate = deso.pstate;
    }
    let { deltaX, deltaY } = event;
    if (event.ctrlKey) {
        const key = (deltaY < 0)
            ? 'scaleUp'
            : ((deltaY > 0) ? 'scaleDown' : 'nop');
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (event.shiftKey) {
        const key = (deltaX < 0) ? 'scrollLeft' : ((deltaX > 0) ? 'scrollRight' : 'nop');
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    }else if (deltaX !== 0 && deltaY === 0) {
        const key = (deltaX < 0) ? 'scrollLeft' : 'scrollRight';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (deltaX === 0 && deltaY !== 0) {
        const key = (deltaY < 0) ? 'scrollUp' : 'scrollDown';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    }
}


/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<string, Handler>} eventHandler 
 * @param {WheelEvent} event 
 */
function linuxWheel(element, pstate, deso, eventHandler, event) {
    if (pstate === undefined)  {
        pstate = deso.pstate;
    }
    const { deltaX, deltaY } = event;
    if (event.ctrlKey) {
        const key = (deltaY < 0)
            ? 'scaleUp'
            : ((deltaY > 0) ? 'scaleDown' : 'nop');
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (event.shiftKey) {
        const key = (deltaY < 0) ? 'scrollLeft' : ((deltaY > 0) ? 'scrollRight' : 'nop');
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (deltaX !== 0 && deltaY === 0) {
        const key = (deltaX < 0) ? 'scrollLeft' : 'scrollRight';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    } else if (deltaX === 0 && deltaY !== 0) {
        const key = (deltaY < 0) ? 'scrollUp' : 'scrollDown';
        if (eventHandler[key].fn(pstate)) {
            deso.render({ type: 'action' });
        }
        event.preventDefault();
    }
}

function registerWheelEvent(element, pstate, deso, eventHandler) {
    const platform = navigator.platform;
    if (platform.startsWith('Win')) {
        return event => windowsWheel(element, pstate, deso, eventHandler, event);
    } else if (platform.startsWith('Linux')) {
        return event => linuxWheel(element, pstate, deso, eventHandler, event);
    } else if (platform.startsWith('Mac')) {
        return event => macOsWheel(element, pstate, deso, eventHandler, event);
    } else {
        throw Error('不支持的操作系统！');
    }
}

export default registerWheelEvent;